local ADDON_NAME, dssk = ...


local KEY_PATTERN = "(" .. strrep(dssk.B64_CHAR_CLASS, 8) .. ")(.)(.)"
local ONE_KEY = "^" .. KEY_PATTERN .. "$"

local function send_key_static()
  dssk:send_my_key()
end

local function request_keys_static()
  if dssk.chat_connected and dssk.player.guild then
    C_ChatInfo.SendAddonMessage(dssk.ADDON_CHAT_PREFIX, dssk.AC_GET_KEYS,
    "GUILD")
  end
end

function dssk:init_addon_chat()
  if not self.chat_connected then
    self.chat_connected =
      C_ChatInfo.RegisterAddonMessagePrefix(self.ADDON_CHAT_PREFIX)
  end
  if self.chat_connected then
    C_Timer.After(self.AC_PRE_GREET_DELAY, request_keys_static)
  end
end

function dssk:handle_addon_chat(text, channel, sender_name)
  -- we only use guild channel addon chat at the moment
  self.lcm = text
  if not self.settings.key_sync or channel ~= "GUILD" then
    return
  end
  if text == self.AC_GET_KEYS then
    -- a request for key sync
    local now = GetServerTime()
    if not self.sendkey_cooldown or self.sendkey_cooldown < now then
      self.sendkey_cooldown = now + self.AC_BROADCAST_COOLDOWN
      -- random delay so we don't all respond at the same moment
      local delay = self.AC_BROADCAST_DELAY + random() * 3.0
      C_Timer.After(delay, send_key_static)
    end
  else
    -- not a command so should be a key
    local guid, map, level = strmatch(text, ONE_KEY)
    if guid and map and level then
      local unit_info = self:guid_info(guid)
      -- if we know of this character
      if unit_info and unit_info.name and unit_info.realm then
        -- make sure they are who they say they are
        if sender_name ~= format("%s-%s", unit_info.name, unit_info.realm) then
          return
        end
        map = string.byte(map, 1) + self.MAP_TRANSMISSION_OFFSET
        level = string.byte(level, 1) - 1
        if level > 0 then
          self:add_guild_key_from_short_guid(guid, map, level)
        end
      end
    end
  end
end

function dssk:send_my_key()
  if not self.chat_connected or not self.player.guild or
      not self.settings.key_sync then
    return
  end
  local db_entry = self:update_player_db_entry()
  local guid = self.player_short_guid
  local map = db_entry and db_entry.key_map_id or nil
  local level = db_entry and db_entry.key_level or 0
  if map then
    map = map - self.MAP_TRANSMISSION_OFFSET
  else
    map = 1
  end
  C_ChatInfo.SendAddonMessage(self.ADDON_CHAT_PREFIX,
    format("%s%s", guid, strchar(map, level + 1)),
    "GUILD")
end
