local ADDON_NAME, dssk = ...

dssk.REQUIRED_LEVEL = 60
dssk.KEYSTONE_LINK_COLOUR = "ffa335ee"
dssk.KEYSTONE_ITEM_ID = 180653
-- pattern to find map_id and key_level in a keystone link
dssk.KEYSTONE_MATCH_PATTERN = "^|c%x%x%x%x%x%x%x%x|Hkeystone:" ..
  dssk.KEYSTONE_ITEM_ID .. ":(%d+):(%d+):%d+:%d+:%d+:%d+|h%[[^|%]]*%]|h|r$"
-- pattern to find the extra data portion of an item style keystone link
dssk.KEYSTONE_ITEM_MATCH_PATTERN = "^|c%x%x%x%x%x%x%x%x|Hitem:" ..
  dssk.KEYSTONE_ITEM_ID .. strrep(":%d*", 12) ..
  ":(%d+):([%d:]*)|h%[[^|%]]*%]|h|r$"
dssk.CHAT_COOLDOWN_SECONDS = 60
dssk.BANG_KEYS_LISTEN_SECONDS = 20

dssk.B64_CHAR_CLASS = "[A-Za-z0-9+/]"

-- Addon chat for syncing key lists
dssk.ADDON_CHAT_PREFIX = "dsskbeta2"
-- seconds after receiving broadcast request before we send our keys
dssk.AC_BROADCAST_DELAY = 10
-- seconds after joining chat before we say "hi" and request keys
dssk.AC_PRE_GREET_DELAY = dssk.AC_BROADCAST_DELAY * 2
-- seconds after sending keys before we'll next reply
dssk.AC_BROADCAST_COOLDOWN = 300

dssk.AC_GET_KEYS = "!"

-- 'first' EU reset, 2020-09-09 07:00
-- in seconds since epoch
dssk.BASE_RESET_TIME_EU = 1599634800
-- other realms, 2020-09-08 15:00
dssk.BASE_RESET_TIME_OTHER = dssk.BASE_RESET_TIME_EU - 16 * 60  * 60
-- length of a week in seconds
dssk.WEEK_SECONDS = 7 * 24 * 60 * 60
-- the furthest EU server local time can be ahead of UTC (with a little margin)
dssk.MAX_EU_UTC_OFFSET = 2 * 65 * 60

dssk.MAPS = {
  { id=244, name="Atal'Dazar" },
  { id=245, name="Freehold" },
  { id=246, name="Tol Dagor" },
  { id=247, name="The MOTHERLODE!!" },
  { id=248, name="Waycrest Manor" },
  { id=249, name="Kings' Rest" },
  { id=250, name="Temple of Sethraliss" },
  { id=251, name="The Underrot" },
  { id=252, name="Shrine of the Storm" },
  { id=353, name="Siege of Boralus" },
  { id=369, name="Mechagon - Junkyard" },
  { id=370, name="Mechagon - Workshop" },
  { id=375, name="Mists of Tirna Scithe" },
  { id=376, name="The Necrotic Wake" },
  { id=377, name="De Other Side" },
  { id=378, name="Halls of Atonement" },
  { id=379, name="Plaguefall" },
  { id=380, name="Sanguine Depths" },
  { id=381, name="Spires of Ascension" },
  { id=382, name="Theater of Pain" },
}

dssk.MAPS_BY_ID = {}

-- subtracted from map id to fit relevant map ids in a byte
dssk.MAP_TRANSMISSION_OFFSET = 242

dssk.CLASS_BY_INDEX = {}
dssk.CLASS_BY_FILE = {}
for i=1,GetNumClasses() do
  local name, file = GetClassInfo(i)
  local c = { name = name, file = file, index = i }
  dssk.CLASS_BY_INDEX[i] = c
  dssk.CLASS_BY_FILE[file] = c
end

-- id = Blizzard's number
-- index = our own number
for index, map in ipairs(dssk.MAPS) do
  map.index = index
  dssk.MAPS_BY_ID[map.id] = map
end

local version = GetAddOnMetadata(ADDON_NAME, "Version")
local major, minor, patch = strsplit(".", version)

dssk.VERSION = {
  as_string = version,
  major = major,
  minor = minor,
  patch = patch,
}
