local ADDON_NAME, dssk = ...

DidSomebodySayKeys_ListEntryMixin = {}
local list_entry = DidSomebodySayKeys_ListEntryMixin

function list_entry:placeholder()
end

function list_entry:convert_to_guild()
  self.faction:Hide()
  self.server:SetWidth(132)
  self.best:Hide()
end

function list_entry:on_click(button)
  if dssk.frame then
    dssk.frame:show_char_detail(self.char_data)
  end
end

local my_char_list = dssk.my_char_list
local guild_char_list = dssk.guild_char_list

local MY_CHARS_COLUMN_INFO = {
  { title = "Realm", width = 100, sort = my_char_list.col.realm },
  { title = "Side", width = 44, sort = my_char_list.col.faction },
  { title = "Class", width = 44, sort = my_char_list.col.class },
  { title = "Name", width = 150, sort = my_char_list.col.name },
  { title = "Key", width = 36, sort = my_char_list.col.key_level },
  { title = "Dungeon", width = 157, sort = my_char_list.col.map_name },
  { title = "Vault", width = 60, sort = my_char_list.col.reward },
}

local GUILD_COLUMN_INFO = {
  { title = "Realm", width = 138, sort = guild_char_list.col.realm },
  { title = "Class", width = 44, sort = guild_char_list.col.class },
  { title = "Name", width = 154, sort = guild_char_list.col.name },
  { title = "Key", width = 36, sort = guild_char_list.col.key_level },
  { title = "Dungeon", width = 217, sort = guild_char_list.col.map_name },
}

DidSomebodySayKeys_DisplayMixin = {}
local display_frame = DidSomebodySayKeys_DisplayMixin

function display_frame:setup_subframe(subframe)
  local headers = subframe.frame.column_headers
  headers:LayoutColumns(subframe.info)
  headers.sortingFunction = function(_, column_index)
    subframe.list:sort_by(subframe.info[column_index].sort)
    self:update_list(subframe)
  end
  local scroll_frame = subframe.frame.scroll_frame
  scroll_frame.update = function()
    self:update()
  end
  scroll_frame.scrollBar.doNotHide = true
  scroll_frame.scrollBar:SetValue(0)
end

function display_frame:on_load()
  dssk.frame = self

  self:SetTitle("Did Somebody Say Keys?")
  self:SetPortraitToAsset(GetItemIcon(dssk.KEYSTONE_ITEM_ID))

  self.subframes = {
    { frame=self.my_keys_frame, info=MY_CHARS_COLUMN_INFO, list=my_char_list },
    { frame=self.guild_keys_frame, info=GUILD_COLUMN_INFO, list=guild_char_list,
      convert_buttons=true },
  }

  for _, v in pairs(self.subframes) do
    self:setup_subframe(v)
  end
  PanelTemplates_SetNumTabs(self, #self.subframes)
  PanelTemplates_SetTab(self, 1)
  self.cur_subframe = self.subframes[1]
end

function display_frame:on_show()
  PlaySound(SOUNDKIT.IG_CHARACTER_INFO_OPEN)
  self:update_list(self.cur_subframe)
end

function display_frame:on_hide()
  PlaySound(SOUNDKIT.IG_CHARACTER_INFO_CLOSE);
  self.detail_frame:Hide()
end

function display_frame:open_settings_menu()
  dssk:open_settings_menu()
end

function display_frame:update_list(subframe)
  local scroll_frame = subframe.frame.scroll_frame
  if not scroll_frame.buttons then
    HybridScrollFrame_CreateButtons(scroll_frame,
      "DidSomebodySayKeys_ListEntryTemplate", 0, 0)
    if subframe.convert_buttons then
      local buttons = scroll_frame.buttons
      for _, b in pairs(buttons) do
        b:convert_to_guild()
      end
    end
  end

  local offset = HybridScrollFrame_GetOffset(scroll_frame)
  local buttons = scroll_frame.buttons

  local height = buttons[1]:GetHeight()

  -- make sure we pull the list from a current db
  dssk:check_db()
  local chars = subframe.list:get_list()
  local total_height = height * #chars
  local used_height = 0

  for i, b in ipairs(buttons) do
    if i + offset <= #chars then
      local char = chars[i + offset]
      b.char_data = char
      b.server:SetText(char.realm)
      if char.faction == "Horde" then
        b.faction:SetTexture("Interface\\Icons\\INV_BannerPVP_01")
      else
        b.faction:SetTexture("Interface\\Icons\\INV_BannerPVP_02")
      end
      local class = char.class or "WARRIOR"
      b.class:SetTexCoord(unpack(CLASS_ICON_TCOORDS[class]))
      local color = RAID_CLASS_COLORS[class];
      b.name:SetTextColor(color.r, color.g, color.b)
      b.name:SetText(char.name)
      b.key_level:SetText(char.key_level or "--")
      b.key_map:SetText(char.map_name or "--")

      -- select chest
      if char.reward_available then
        b.best.icon:Hide()
        b.best.icon_glow:Show()
        b.best.icon_avail:Show()
        b.best.glow_anim:Play()
      else
        b.best.icon_glow:Hide()
        b.best.icon_avail:Hide()
        b.best.glow_anim:Stop()
        if char.reward_level and char.reward_level > 0 then
          b.best.icon:Show()
        else
          b.best.icon:Hide()
        end
      end

      if char.reward_level and char.reward_level > 0 then
        b.best.text:SetText(char.reward_level)
      else
        b.best.text:SetText("--")
      end

      b:Show()
      used_height = used_height + height
    else
      b:Hide()
    end
  end
  HybridScrollFrame_Update(scroll_frame, total_height, used_height)
end

function display_frame:update()
  self:update_list(self.cur_subframe)
end

function display_frame:tab_button_click(id)
  PanelTemplates_SetTab(self, id)
  for _, v in pairs(self.subframes) do
    v.frame:Hide()
  end
  self.detail_frame:Hide()
  self.cur_subframe = self.subframes[id]
  self.cur_subframe.frame:Show()
  self:update_list(self.cur_subframe)
  PlaySound(SOUNDKIT.IG_CHARACTER_INFO_TAB)
end

function display_frame:show_char_detail(char_data)
  if char_data then
    self.detail_frame:show_char(char_data)
  end
end
