local ADDON_NAME, dssk = ...

local events = {}

function events:ADDON_LOADED(name)
  if name == ADDON_NAME then
    dssk:load_vars()
    print(dssk.intro_text)
  end
end

function events:MYTHIC_PLUS_CURRENT_AFFIX_UPDATE()
  dssk.affixes = C_MythicPlus.GetCurrentAffixes()
end

function events:PLAYER_LOGIN()
  dssk:player_login()
end

function events:PLAYER_LEVEL_UP(level)
  dssk:player_level_update(level)
end

function events:CHAT_MSG_PARTY(msg, sender_name, ...)
  local guid = select(10, ...)
  dssk:handle_chat(msg, 'party', sender_name, guid, true)
end

function events:CHAT_MSG_PARTY_LEADER(msg, sender_name, ...)
  local guid = select(10, ...)
  dssk:handle_chat(msg, 'party', sender_name, guid, true)
end

function events:CHAT_MSG_RAID(msg, sender_name, ...)
  local guid = select(10, ...)
  dssk:handle_chat(msg, 'raid', sender_name, guid, true)
end

function events:CHAT_MSG_RAID_LEADER(msg, sender_name, ...)
  local guid = select(10, ...)
  dssk:handle_chat(msg, 'raid', sender_name, guid, true)
end

function events:CHAT_MSG_GUILD(msg, sender_name, ...)
  local guid = select(10, ...)
  dssk:handle_chat(msg, 'guild', sender_name, guid, false)
end

function events:CHAT_MSG_ADDON(prefix, text, channel, sender_name)
  if prefix == dssk.ADDON_CHAT_PREFIX then
    dssk:handle_addon_chat(text, channel, sender_name)
  end
end

function events:ENCOUNTER_LOOT_RECEIVED(enc_id, item_id, item_link, quant,
  player_name)
  if item_id == dssk.KEYSTONE_ITEM_ID then
    dssk:key_looted(player_name, item_link)
  end
end

function events:PLAYER_GUILD_UPDATE(unit)
  if unit == 'player' then
    dssk:player_guild_update()
  end
end

function events:CHALLENGE_MODE_COMPLETED()
  -- delay this a little for the updated key to reach us
  C_Timer.After(2, function()
    dssk:update_key(true)
    -- Event MYTHIC_PLUS_NEW_WEEKLY_RECORD doesn't seem to work at the moment.
    -- So instead we request rewards here to update ourselves.
    C_MythicPlus.RequestRewards()
  end)
end

function events:CHALLENGE_MODE_START()
  dssk:update_key()
end

function events:CHALLENGE_MODE_MAPS_UPDATE()
  dssk:update_rewards()
end

-- Auto-slot Keystone
function events:CHALLENGE_MODE_KEYSTONE_RECEPTABLE_OPEN()
  if dssk.settings.auto_slot then
    local bag, slot = dssk:find_key_in_bag()
    if bag then
      PickupContainerItem(bag, slot)
      C_ChallengeMode.SlotKeystone()
    end
  end
end

-- Looting vault
function events:WEEKLY_REWARDS_UPDATE()
  -- delay a little for inventory update
  C_Timer.After(3, function()
    dssk:update_key()
    dssk:update_rewards()
  end)
end

-- Downgrading key at Keystone Trader
function events:ITEM_CHANGED(old_link, new_link)
  local map, level = dssk:link_to_map_level(new_link)
  if map then
    dssk:set_my_key(map, level)
  end
end

DidSomebodySayKeys_EventMixin = {}
local frame = DidSomebodySayKeys_EventMixin

function frame:on_load()
  for k, _ in pairs(events) do
    self:RegisterEvent(k)
  end
end

function frame:on_event(event, ...)
  events[event](self, ...)
end
