local ADDON_NAME, dssk = ...

local i_to_b64 = {
  [0] = 'A',
  'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
  'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
  'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
  'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/',
}

local b64_to_i = {}
for k, v in pairs(i_to_b64) do
  b64_to_i[strbyte(v)] = k
end

-- Turn a regular player GUID into a short form version for transmission.
-- Our short form GUIDs are currently the 16bit server id and 32 bit player id
-- base64 encoded.
local PLAYER_GUID_PATTERN = "^Player%-(%x+)%-(%x+)$"
local guid_squish_working_table = {}
function dssk:squish_player_guid(longform_guid)
  local server_id, player_id = strmatch(longform_guid, PLAYER_GUID_PATTERN)
  server_id, player_id = tonumber(server_id, 16), tonumber(player_id, 16)
  if not server_id or not player_id then
    -- print("guid match failed")
    return nil
  end
  local wt = guid_squish_working_table
  wt[1] = bit.rshift(server_id, 10)
  wt[2] = bit.band(bit.rshift(server_id, 4), 0x3f)
  wt[3] = bit.lshift(bit.band(server_id, 0xf), 2) + bit.rshift(player_id, 30)
  wt[4] = bit.band(bit.rshift(player_id, 24), 0x3f)
  wt[5] = bit.band(bit.rshift(player_id, 18), 0x3f)
  wt[6] = bit.band(bit.rshift(player_id, 12), 0x3f)
  wt[7] = bit.band(bit.rshift(player_id, 6), 0x3f)
  wt[8] = bit.band(player_id, 0x3f)
  for k, v in pairs(wt) do
    wt[k] = i_to_b64[v]
  end
  return table.concat(wt)
end

-- Turn a short form GUID back into a normal Blizzard format GUID.
function dssk:expand_player_guid(shortform_guid)
  -- validate format
  if #shortform_guid ~= 8 or
      not strmatch(shortform_guid, "^[A-Za-z0-9+/]+$") then
    return nil
  end
  local wt = guid_squish_working_table
  for i = 1,8 do
    wt[i] = b64_to_i[strbyte(shortform_guid, i)]
  end
  local server_id = bit.lshift(wt[1], 10) + bit.lshift(wt[2], 4) +
    bit.rshift(wt[3], 2)
  local player_id = bit.lshift(bit.band(wt[3], 0x3), 30) +
    bit.lshift(wt[4], 24) + bit.lshift(wt[5], 18) +
    bit.lshift(wt[6], 12) + bit.lshift(wt[7], 6) + wt[8]
  return format("Player-%X-%08X", server_id, player_id)
end

function dssk:capture_guid(guid)
  if not self.guid_cache then
    return
  end
  local _, class, _, race, sex, name, realm = GetPlayerInfoByGUID(guid)
  if class then
    if not realm or realm == "" then
      realm = self:get_realm_name()
    end
    local entry = self.guid_cache[guid]
    if not entry then
      entry = { short = self:squish_player_guid(guid) }
      self.guid_cache[guid] = entry
      self.short_guid_cache[entry.short] = entry
    end
    entry.class = class
    entry.race = race
    entry.sex = sex
    entry.name = name
    entry.realm = realm
    entry.seen = GetServerTime()
    --print(format("Captured %s-%s, a %s %s (%s)", name, realm, race, class, entry.short))
  end
end

-- Get cached info from short-from guid.
-- Marks the info as recently read,
-- so we don't clean up old data we're still regularly using
function dssk:guid_info(short_guid)
  local entry = self.short_guid_cache[short_guid]
  if entry then
    entry.read = GetServerTime()
  end
  return entry
end

-- load GUID Cache from Saved Variables
function dssk:load_guid_cache()
  DidSomebodySayKeys_GuidCache = DidSomebodySayKeys_GuidCache or {}
  self.guid_cache = DidSomebodySayKeys_GuidCache

  -- Trim the cache, removing anyone we've not seen for a couple of months
  local target = GetServerTime() - 60 * 60 * 24 * 60
  local cache = self.guid_cache
  for k, v in pairs(cache) do
    if (not v.seen or v.seen < target) and (not v.read or v.read < target) then
      cache[k] = nil
    end
  end

  -- Reference GUID info by short form.
  self.short_guid_cache = {}
  for _, v in pairs(cache) do
    self.short_guid_cache[v.short] = v
  end
end
