local ADDON_NAME, dssk = ...

-- Setup globals for interacting with keybinds etc.

DidSomebodySayKeys = {
  toggle_window = function(self)
    dssk:toggle_window()
  end,
  dssk = dssk
}

BINDING_HEADER_DIDSOMEBODYSAYKEYS = ADDON_NAME
BINDING_NAME_DIDSOMEBODYSAYKEYS_TOGGLEWINDOW = "Toggle Window"

function dssk:open_settings_menu()
  -- call it twice, sometimes the first call only opens the menu
  -- and doesn't jump to the category
  InterfaceOptionsFrame_OpenToCategory(dssk.settings_frame)
  InterfaceOptionsFrame_OpenToCategory(dssk.settings_frame)
end

dssk.intro_text = [[
DidSomebodySayKeys loaded:
"/dssk" to toggle window or "/dssk help" for more information.
]]

dssk.help_text = [[
Did Somebody Say Keys?
An addon for keeping track of your weekly Mythic Keystones.
Records a character's Keystone when one is looted (or on login).
Will automatically reply with your key when someone says "!keys".
Also tracks each character's highest run of the week.

Commands:
  "/dssk" - toggle window
  "/dssk settings" - settings window.
  "/dssk reset" - delete this week's stored data.

Window toggle can be bound to a key at:
Game Menu > Key Bindings > AddOns > DidSomebodySayKeys
]]

local function print_keys()
  dssk:check_db()
  local characters = dssk.my_char_list:get_list()
  local hit = false
  for _, v in pairs(characters) do
    if v.key_level then
      local reward_str = ""
      if v.reward_available then
        reward_str = "Cache available"
      elseif v.reward_level and v.reward_level > 0 then
        reward_str = "Done " .. v.reward_level
      end
      print(string.format("%s %s %s",
      v.name,
      dssk:make_keystone_link(v.key_map_id, v.key_level),
      reward_str
      ))
      hit = true
    end
  end
  if not hit then
    print("No keys")
  end
end

local commands = {
  reset = function() dssk:reset_db() end,
  settings = function() dssk:open_settings_menu() end,
  toggle = function() dssk:toggle_window() end,
  dump = function() print_keys() end,
  help = function() print(dssk.help_text) end,
}

local function slash_handler(params, editbox)
  local iter = params:gmatch("%s*(%S+)")
  local cmd = iter()

  if not cmd then
    commands.toggle()
  else
    if commands[cmd] then
      commands[cmd](iter)
    else
      print("dssk: unrecognised command")
    end
  end
end

SLASH_DIDSOMEBODYSAYKEYS1 = "/dssk"
SlashCmdList["DIDSOMEBODYSAYKEYS"] = slash_handler
